;Chong Carl
;SPARC ISR.ASM

;All interrupt handlers reside here
;PORTA outputs the chirping
;PORTB inputs the light condition
;PORTC outputs the servo control PWM
;PORTD inputs the switches to debug the IR light sensor

rjmp	INIT_Machine	;Restart vector, points to the beginning of the code. 
reti
reti
reti
reti
reti
rjmp	TIM1_OVF	;Timer 1 overflow
rjmp	TIM0_OVF	;Timer 0 overflow
reti
reti
reti
reti
reti

;***************************************************************
;Timer 1 gives us servo output timings from 1mS to 2mS, and allows
;the CPU to go do other things, rather than sitting there spinning 
;NOPS. It's a 16 bit counter, so I start it with a base value of 1mS,
;plus the servo width control byte (SERVO_X) We then have 1-2mS pulses,
;with full 8 bit resolution (4uS)

;Each servo out is activated sequentially in the main code. The activation
;code loads a value into Timer1, and sets the prescaler to /8, and sets the
;desired servo bit high. This int occurs when the servo's time expires, and
;we just shut off the output, and stop the timer.

TIM1_OVF:
	in	TTEMP2,SREG	;Saves the status register

	;**************************************************
	;Support for the R/C servo driver in SERVO.ASM
	;**************************************************

	;No matter who was running, shut off all servo outputs
	ldi	TTEMP,$00	;
	out	PORTC,TTEMP	;

	in	TTEMP,TCCR1B	;Get the timer control register
	andi	TTEMP,$F8	;Set the prescaler to "off"
	out	TCCR1B,TTEMP	;Make it so.

	;This test could instead mask and look for a particular
	;bit to reduce the number of active servos.

	lds	TTEMP,Servo_Control	;

	;To make the servos run from 1 to 8, enable this
	lsl	TTEMP			;Set next servo to run
	brcc	Servo_TB		;If no carry then done, else
	ldi	TTEMP,$01		;start over on servo 1(0)

Servo_TB:
	sts	Servo_Control,TTEMP	;Set the next active servo bit on.

TIM1O_EXIT:
	out	SREG,TTEMP2		;restore the status register
	reti				;
;***************************************************************
;	Main 1mS Opsys tick
TIM0_OVF:

	in	TTEMP3,SREG	;Saves the status register

	;**************************************************
	;Reload timer 0 now to preserve accuracy
	;**************************************************
	ldi	TTEMP,T0DIV	;Reload early, avoid the latency
	out	TCNT0,TTEMP	;

	;**************************************************
	;Support for the R/C servo driver in SERVO.ASM
	;**************************************************

T0_RC:	lds	TTEMP,Frame_Delay;Get delay timer from SRAM
	and	TTEMP,TTEMP	;Is it zero?
	breq	T0_RC_Done	;leave it alone if so
	dec	TTEMP		;Otherwise dec
	sts	Frame_Delay,TTEMP;and store

T0_RC_Done: ;Dummy Label

TIM0_EXIT:
	out	SREG,TTEMP3	;Restore the status register
	reti			